const util = require('util');
const Node = require('./node');

module.exports = function LinkedList() {
    return {
        headNode: null,

        append(value) {
            if (!this.headNode) {
                this.headNode = Node(value);
            } else {
                let currentNode = this.headNode;
                while (currentNode.next) {
                    currentNode = currentNode.next;
                }
                currentNode.next = Node(value);
            }
        },

        prepend(value) {
            if (!this.headNode) {
                this.headNode = Node(value);
            } else {
                const temp = this.headNode;
                this.headNode = Node(value);
                this.headNode.next = temp;
            }
        },

       size() {
        let i = 0;
        if (this.headNode) {
            i = 1;
            let currentNode = this.headNode;
            while (currentNode.next) {
                i++;
                currentNode = currentNode.next;
            }
        }
        return i;
       },

       head() {
        return this.headNode.value;
       },

       tail() {
        let currentNode = this.headNode;
        while(currentNode.next) {
            currentNode = currentNode.next;
        }
        return currentNode;
       },

       at(index) {
        if (index === 0) return this.headNode.value;

        let currentNode = this.headNode;
        for (let i = 0; i < index; i++) {
            if (currentNode) {
                currentNode = currentNode.next;
            }
        }
        if (currentNode) {
            return currentNode.value;
        } else {
            return 'No node at that index!';
        }     
       },

       pop() {
        let currentNode = this.headNode;
        let prevNode;
        while (currentNode.next) {
            prevNode = currentNode;
            currentNode = currentNode.next;
        }
        prevNode.next = null;
       },

       contains(value) {
        let currentNode = this.headNode;

        while (currentNode && currentNode.value !== value) {
            currentNode = currentNode.next;
        }

        if (currentNode && currentNode.value === value) {
            return true;
        } else {
            return false;
        }
       },

       find(value) {
        let currentNode = this.headNode;

        let i = 0;
        while (currentNode) {
            if (currentNode.value === value) {
                return i;
            } else {
                currentNode = currentNode.next;
                i++;
            }
        }
        
        return null;
       },

       toString() {
        let string = '';

        let currentNode = this.headNode;

        while (currentNode) {
            string += `( ${currentNode.value} ) -> `;
            currentNode = currentNode.next;
        }
        string += 'null'

        return string;
       },

       insertAt(value, index) {
        if (index === 0) {
            const temp = this.headNode;
            this.headNode = Node(value);
            this.headNode.next = temp;
        }
        let currentNode = this.headNode;
        let prevNode;

        for (let i = 0; i < index; i++) {
            if (currentNode) {
                prevNode = currentNode;
                currentNode = currentNode.next;
            }
        }
        const temp = currentNode;
        currentNode = Node(value);
        if (prevNode) prevNode.next = currentNode;
        currentNode.next = temp;
       },

       removeAt(index) {
        if (index === 0) {
            this.headNode = this.headNode.next;
        } else {
            let currentNode = this.headNode;
            let prevNode;
            let i = 0;
            while (i < index) {
                prevNode = currentNode;
                if (currentNode) currentNode = currentNode.next;
                i++;
            }
            prevNode.next = currentNode.next;
        }
       }
    }
}
